;________________________________________________________________________
;
;                              R 9 1 8 _ P I C . A S M
;________________________________________________________________________
;
; WMR918_PIC Main code - by Max IK3SVW - July 2001
; Based on original u2k package by Byon Garrabrant, N6BG
;________________________________________________________________________

	LIST	P=PIC16F84
	include	"P16F84.INC"
	__CONFIG _HS_OSC & _CP_OFF & _PWRTE_ON & _WDT_OFF

;________________________________________________________________________

	cblock	H'0C'		; system variables
		beacon_counter
		wx_counter
		temp
		temp2
		alldecoded
		auxdecoded
		nextXmitL
	endc

; #define 	TEST		; flash on reset
acquireTime	equ	5	; time to acquire all sensors

;________________________________________________________________________

#define	PTT_IN		PORTA, 3
#define	PTT_OUT		PORTA, 2
#define	VALID_OUT	PORTB, 0
#define	VOX_IN		PORTB, 3
#define	MODE_IN		PORTA, 4
#define	OPTION_IN	PORTB, 4
;________________________________________________________________________

	ORG	0		; reset vector
	call	serialInit
	call	Tone_Init
	clrf	auxdecoded
	goto	main
;________________________________________________________________________

	ORG	0x04

	include	"MX614.ASM"	; include MX614 Modem code
	include	"TABLES.ASM"	; tables need to stay low in memory
	include	"OPTIONS.ASM"	; user options
	include "AX25.ASM"	; AX-25 routines
	include	"BCD.ASM"	; include BCD routine
	include	"WMR918CT.ASM"	; include WMR-918 code
	include "SERIAL.ASM"	; include serial input code
;________________________________________________________________________

Packet_Send_String
	movfw	hex1
	call	Get_Option
	iorlw	0
	skpnz
	 return
	call	AX25_Send_Data
	incf	hex1, F
	goto	Packet_Send_String
;________________________________________________________________________

sendPrediction
	btfss	MODE_IN		; check for user predicion option
	 return
	movlw	' '
	call 	AX25_Send_Data
	movlw	'P'
	call 	AX25_Send_Data
	movlw	'r'
	call 	AX25_Send_Data
	movlw	'e'
	call 	AX25_Send_Data
	IFDEF	TEST
	movlw	'v'
	call 	AX25_Send_Data
	ENDIF
	call 	AX25_Send_Dot
	btfsc	barotrend,1	; choose baro prediction user label
	 movlw	OPT_BARO1_OFFSET
	btfsc	barotrend,2
	 movlw	OPT_BARO2_OFFSET
	btfsc	barotrend,3
	 movlw	OPT_BARO0_OFFSET
	btfsc	barotrend,0
	 movlw	OPT_BARO3_OFFSET
	call	Get_Option	; send barometer prediction
	movwf	hex1
	goto	Packet_Send_String
;________________________________________________________________________

SendBeacon
	call	AX25_Send_Start
	movlw	'>'
	call 	AX25_Send_Data	; send status char.
	btfsc	alldecoded, 3	; if baro decoded send prediction
	 call	sendPrediction
	movlw	OPT_STAT_STRING_OFFSET
	call	Get_Option	; load user status string
	movwf	hex1
	call	Packet_Send_String
	goto	AX25_Send_End
;________________________________________________________________________

BigDelay			; long delay routine
	clrf	hex1
bigDly1	clrf	hex2
bigDly2	decfsz	hex2, f
	 goto bigDly2
	decfsz	hex1, f
	 goto	bigDly1
	return
;________________________________________________________________________

waitDCD_PTT			; check for VOX high
	btfsc	VOX_IN
	 goto	waitDCD_PTT
	call	BigDelay
waitDCD1			; recheck a while later to avoid false low
	btfsc	VOX_IN
	 goto	waitDCD1

	btfss	PTT_IN		; check if external PTT active
	 goto	waitDCD_PTT

	bsf	PTT_OUT		; all clear for transmit, press PTT
	return			
;________________________________________________________________________
	
resRtot	movfw	raintotL	; Reset rain counters
	movwf	rain1hL
	movwf	offsetRtotL
	movfw	raintotH
	movwf	rain1hH
	movwf	offsetRtotH
	return
;________________________________________________________________________
	
main	bsf	STATUS, RP0	; Select Bank 1
	bcf	PTT_OUT		; make PTT an output
	bcf	VALID_OUT	; make Valid Led an output
	bcf	STATUS, RP0	; Select Bank 0
	bcf	PTT_OUT		; release PTT

	clrf	alldecoded	; clear various flags
	movlw	D'1'
	movwf	beacon_counter	; initialize status beacon counter
	movlw	acquireTime
	movwf	wx_counter	; initialize wx beacon counter
	
	IFDEF	TEST
	movwf	temp
flash	bsf	VALID_OUT	; some valid led flashes on reset
	call	BigDelay
	bcf	VALID_OUT
	call	BigDelay
	decfsz	temp, f
	 goto	flash
	ENDIF
;________________________________________________________________________

mainLoop
	call	serialReceive	; receive serial byte
	movwf	checksum
	xorlw	H'FF'
	skpz
	 goto	mainLoop
	call	serialReceiveCk	; receive serial byte
	xorlw	H'FF'
	skpz
	 goto mainLoop
	call	serialReceiveCk	; receive serial byte
	movwf	temp
	xorlw	H'00'		; look at result
	skpnz			; if not a group code continue
	 goto	Decode00	; else decode data
	movfw	temp
	xorlw	H'01'		; look at result
	skpnz			; if not a group code continue
	 goto	Decode01	; decode data
	movfw	temp
	xorlw	H'03'		; look at result
	skpnz			; if not a group code continue
	 goto	Decode03	; decode data
	movfw	temp
	xorlw	H'05'		; look at result
	skpnz			; if not a group code continue
	 goto	Decode05	; decode data
	movfw	temp
	xorlw	H'06'		; look at result
	skpnz			; if not a group code continue
	 goto	Decode06	; decode data
	movfw	temp
	xorlw	H'0E'		; look at result
	skpnz			; if not a group code continue
	 goto	allChk1		; exit loop
	movfw	temp
	xorlw	H'0F'		; look at result
	skpnz			; if not a group code continue
	 goto	Decode0F	; decode data
;________________________________________________________________________

allChk	bcf	VALID_OUT	; turn off valid LED
	goto	mainLoop

allChk1	call	convertWX	; convert and fix wx units
	movfw	auxdecoded
	andlw	H'BF'
	btfss	alldecoded, 6	; is pic-e just powered on ?
	 goto	alchk2
	iorwf	alldecoded, f	; store decode flags
	
	btfss	auxdecoded, 4	; check hour chime
	 goto	restim1
	movlw	H'50'
	movwf	alldecoded	; if hour changes, clear decode flags
	movlw	acquireTime
	movwf	wx_counter	; time to let all sensors be decoded
	movfw	raintotL	; if hour changes, reset r
	movwf	rain1hL
	movfw	raintotH
	movwf	rain1hH
 	call	Shift4Baro	; shift baro on 4-hour history
	movfw	timeH		; check day change
	skpnz
	 call resRtot		; if day changes, reset P & r

restim1	
	btfss	OPTION_IN	; check if forced P & r reset
	 goto	restim2

	movfw	raintotL	; check for total rain value
	skpz
	 goto	restim3
	movfw	raintotH
	skpnz
restim2	 call	resRtot		; if total rain is 0, reset P & r

restim3 btfsc	alldecoded, 4	; disable immediate bcn on hour chime
	 goto	alchk2
	movlw	OPT_WX_ALERT
	call	Get_Option
	movwf	hex3
	btfss	hex3, 0		; check for 1-minute rain driven flag
	 goto	alchk1
	movfw	raintotL
	xorwf	auxrain, w	; check for rain change in 1 minute
	skpnz
	 goto	alchk1
	movfw	raintotL
	movwf	auxrain
	movlw	D'1'
	movwf	wx_counter
	bsf	VALID_OUT

alchk1	btfss	hex3, 1		; check for 1-minute baro driven flag
	 goto	alchk2
	movfw	baroHex
	xorwf	auxbaro, w	; check for baro change in 1 minute
	skpnz
	 goto	alchk2
	movfw	baroHex
	movwf	auxbaro
	movlw	D'1'
	movwf	wx_counter
	bsf	VALID_OUT

alchk2	clrf	hex3
	decfsz	wx_counter, f	; wx beacon timing
	 goto	bcntest
	btfsc	alldecoded, 6	; power on check
	 goto	alchk3
	movfw	auxdecoded	; if power on store last decode flags
	movwf	alldecoded
	bsf	alldecoded, 6	; reset power on flag
alchk3	bcf	alldecoded, 4	; reset hour chime flag
	movlw	OPT_WX_BEACON_RATE
	call	Get_Option
	movwf	wx_counter	; reset wx beacon counter
	call	waitDCD_PTT
	incf	hex3, f
	call	Packet_Send	; send the wx packet

bcntest	decfsz	beacon_counter, f
	 goto	nobcn
	movlw	OPT_STATUS_RATE
	call	Get_Option
	movwf	beacon_counter	; reset status beacon counter
	btfss	hex3, 0		; check if PTT already pressed
	 call	waitDCD_PTT
	call	SendBeacon	; send the status packet

nobcn	clrf	auxdecoded	; clear last one minute decode flags
	bcf	PTT_OUT		; release PTT
	goto	mainLoop
;________________________________________________________________________

Snd4Dot	call	AX25_Send_Dot	; send dots
Snd3Dot	call	AX25_Send_Dot
Snd2Dot	call	AX25_Send_Dot
	goto	AX25_Send_Dot
;________________________________________________________________________
	
Packet_Send			; Send the wx data packet
	call	AX25_Send_Start
;	call	sendHeader
;
; example target packet:
; =4500.00N/01200.00E_293/006g009t063r000P024b0130h60v+01 Pre.Sunny
;
	movlw	OPT_WX_BEACON_OFFSET
	call	Get_Option	; send wx beacon header
	movwf	hex1
	call	Packet_Send_String

	movf	directionH,w
	call	AX25_Send_Hex_Nibble
	swapf	directionL,w
	call	AX25_Send_Hex
	movlw	'/'
	call 	AX25_Send_Data
	btfsc	alldecoded, 0	; check if wind wad decoded
	 goto	windok
	call	Snd3Dot
	goto	winderr
windok	call 	AX25_Send_Zero
	movf	speedH,w
	call	AX25_Send_Hex
	movlw	'g'
	call 	AX25_Send_Data
	call 	AX25_Send_Zero
	movf	gustH,w
	call	AX25_Send_Hex
winderr	movlw	't'
	call 	AX25_Send_Data
	btfsc	alldecoded, 2	; check if temperature was decoded
	 goto	tempok
	call	Snd3Dot
	goto	temperr
tempok	movlw	'-'
	btfss	alldecoded,7	; check minus sign
	 movlw	'0'
	call 	AX25_Send_Data
	movf	tempL,w
	call	AX25_Send_Hex
temperr	movlw	'r'
	call 	AX25_Send_Data
	btfsc	alldecoded, 1	; check if rain was decoded
	 goto	rainrok
	call	Snd3Dot
	goto	norainP
rainrok	movf	rainrH,w
	call	AX25_Send_Hex_Nibble
	movf	rainrL,w
	call	AX25_Send_Hex
	movf	rainPL,w
	skpz
	 goto	rainPok
	movf	rainPH,w
	skpnz			; skip if P rain is 0
	 goto	norainP
rainPok	movlw	'P'
	call 	AX25_Send_Data
	movf	rainPH,w
	call	AX25_Send_Hex_Nibble
	movf	rainPL,w
	call	AX25_Send_Hex
norainP	movlw	'b'
	call 	AX25_Send_Data
	btfsc	alldecoded, 3	; check if baro was decoded
	 goto	barook
	call	Snd4Dot
	goto	baroerr
barook	movf	baro_seaH,w
	call	AX25_Send_Hex_Nibble
	movf	baro_seaL,w
	call	AX25_Send_Hex
	call 	AX25_Send_Zero
baroerr	movlw	'h'
	call 	AX25_Send_Data
	btfsc	alldecoded, 2	; check if humidity was decoded
	 goto	humok
	call	Snd2Dot
	goto	humerr
humok	movf	humidL,w
	call	AX25_Send_Hex
humerr	btfss	alldecoded,3
	 goto	AX25_Send_End
	movf	speedL_barovar,w
	skpnz			; skip if baro var (v) is zero	
	 goto	noBvar
	movlw	'v'
	call 	AX25_Send_Data
	movlw	'+'
	btfsc	alldecoded,5
	 movlw	'-'
	call 	AX25_Send_Data
	movf	speedL_barovar,w
	call	AX25_Send_Hex
noBvar	call	sendPrediction	; send barometer prediction

	goto	AX25_Send_End
;________________________________________________________________________

	END
;________________________________________________________________________